/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * FontEntry.h: Header for FontEntry.cc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FONTENTRY_H
#define FONTENTRY_H

#include <String.h>

/// Describes a disk-based RawFont.
class FontEntry {
  ///
  void clear(void);
  ///
  void free(void);
  ///
public:
  /// Pathname to load font from
  String name;
  /// Free-form text describing this font
  String description;
  /// Height of glyphs in font
  int height;
  /// Width of glyphs in font
  int width;
  ///
  FontEntry(void) {POFUNC;};
  ///
  ~FontEntry() {POFUNC;};

  /// Add a font list from a named directory
  friend BOOL AddFontsFromDir(const String &s);
  /// Add font lists from a colon-separated list of directories
  friend BOOL AddFontsFromPath(const String &s);
  /// 
  friend String FindBestFont(int, int);

  // For the FontEntrySet class to search and sort
  friend int FontEntryEQ(const FontEntry &a, const FontEntry &b) {return a.name==b.name;};
  friend int FontEntryCMP(const FontEntry &a, const FontEntry &b) {return (a.name<=b.name)?(a.name==b.name)?0:-1:1;};
};

#endif /* FONTENTRY_H */

