/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * RastFunc.h - Rasterises CharCell[]s to various formats
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef RASTFUNC_H
#define RASTFUNC_H

#include "CharMap.h"
#include "RawFont.h"

void RenderCharMap8(const CharMap &map, const CharMap &map2, const RawFont &font, const int cmap[16],
		      UBYTE *fbuf, int sw);

/* Rasterises a CharMap onto a sixteen bit colour display.

  @param map CharMap to rasterise.
  @param font Font to draw with.
  @param cmap Colour values to use.
  @param fbuf Pointer to frame buffer.
  @param sw Width of screen in pixels.
  @param curx X position of cursor.
  @param cury Y position of cursor.
 */

void RenderCharMap16(const CharMap &map, const CharMap &map2, const RawFont &font, const int cmap[16],
		      UWORD *fbuf, int sw);

void RenderCharMap32(const CharMap &map, const CharMap &map2, const RawFont &font, const int cmap[16],
		      ULONG *fbuf, int sw);

#endif /* RASTFUNC_H */
