/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * Serial.cc - serial port driver
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "../config.h"

#ifdef USE_SERIAL

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>

// Networking stuff
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <String.h>

#include "global.h"
#include "ConnectionDriver.h"
#include "Serial.h"

void Serial::clear(void) {
  sock=-1;
  POFUNC;
}
 
void Serial::free(void) {
  if(sock>=0)
    close(sock);
  POFUNC;
}

Serial::Serial(void) {
  POFUNC;
  clear();
}

Serial::~Serial() {
  POFUNC;
  free();
}

Serial *newSerial(String &path) {
  Serial *t;

  if(!(t=new Serial))
    return NULL;

  t->sock=open(path, O_RDWR|O_NONBLOCK);
  
  if(t->sock<0) {
    delete t;
    return NULL;
  }

  fcntl(t->sock, F_SETFL, O_RDWR);

  /* This changes the serial mode to something usable for Viewdata,
   * that is 38400 bps, 8 bits, even parity, 1 stop bit.
   */
  struct termios tios;
  tcgetattr(t->sock, &tios);

  cfmakeraw(&tios);
  cfsetispeed(&tios, B38400);
  cfsetospeed(&tios, B38400);
  tios.c_cflag&=~(CSIZE|PARODD|CSTOPB);
  tios.c_cflag|=PARENB|CS7|CRTSCTS;
  tcsetattr(t->sock, TCSANOW, &tios);
  D("Set port to raw mode, 7E1, 38400");
  return t;
}

int Serial::write(const char *buf, int len) {
  POFUNC;
  D("Writing " << len << " bytes to fd " << sock);
  if(sock<0)
    return -1;
  int l=::write(sock, buf, len);
  D("... write() returned " << l);
  return l;
}

int Serial::read(char *buf, int len) {
  POFUNC;
  D("Reading (at most) " << len << " bytes to fd " << sock);
  if(sock<0)
    return -1;
  int l=::read(sock, buf, len);
  D("... read() returned " << l);
  return l;
}

#endif /* USE_SERIAL */
