/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * Telnet.cc - TCP/IP stream driver
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "../config.h"

#ifdef USE_TELNET

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

// Networking stuff
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <String.h>

#include "global.h"
#include "ConnectionDriver.h"
#include "Telnet.h"

void Telnet::clear(void) {
  sock=-1;
  POFUNC;
}
 
void Telnet::free(void) {
  if(sock>=0) {
    close(sock);
    cerr << "Network connection closed.\n";
  }
  POFUNC;
}

Telnet::Telnet(void) {
  POFUNC;
  clear();
}

Telnet::~Telnet() {
  POFUNC;
  free();
}

int netopen(const String &host, const String &service) {
  int sock;
  struct in_addr defaddr;
  struct hostent *hp, def;
  struct sockaddr_in sin;
  char *alist[1];
  int port;

  if(!(port=atoi(service))) {
    struct servent *sp;
    if(!(sp=getservbyname(service, "tcp"))) {
      cerr << PROGNAME ": tcp/" << service << ": unknown service\n";
      return -1;
    }
    port=ntohs(sp->s_port);
  }

  /* Lookup the hostname */
  if(!(hp=gethostbyname(host))) {
    defaddr.s_addr=inet_addr(host);
    /* Cast stops warning about signed/unsigned conversion */
    if((int)defaddr.s_addr==-1) {
      cerr << PROGNAME ": " << host << ": Host name lookup failure\n";
      return -1;
    }
    (const char *)def.h_name = host;
    def.h_addr_list = alist;
    def.h_addr = (char *)&defaddr;
    def.h_length = sizeof(struct in_addr);
    def.h_addrtype = AF_INET;
    def.h_aliases = 0;
    hp = &def;
  }

  cerr << "Trying "
       << (int)(hp->h_addr[0]&255) << "." << (int)(hp->h_addr[1]&255) << "."
       << (int)(hp->h_addr[2]&255) << "." << (int)(hp->h_addr[3]&255) << "\n";
  
  /* Create the socket */
  sin.sin_family=hp->h_addrtype;
  bcopy(hp->h_addr, (char *)&sin.sin_addr, hp->h_length);
  sin.sin_port=htons(port);
  if((sock=socket(hp->h_addrtype, SOCK_STREAM, 0))<0) {
    cerr << PROGNAME ": " << strerror(errno) << "\n";
    return -errno;
  }

  /* Connect the socket */
  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin))<0) {
    cerr << PROGNAME ": " << strerror(errno) << "\n";
    close(sock);
    return -errno;
  }
    cerr << "Connected.\n";
  return sock;
}

Telnet *newTelnet(String &path) {
  Telnet *t;

  if(!(t=new Telnet))
    return NULL;

  String host;
  String port;

  int idx=path.index(":");

  if(idx>=0) {
    host=path.before(idx);
    port=path.after(idx);
  } else {
    host=path;
    port="telnet";
  }

  t->sock=netopen(host, port);
  if(t->sock<0) {
    delete t;
    return NULL;
  }
    
     return t;
}

int Telnet::write(const char *buf, int len) {
  POFUNC;
  return ::write(sock, buf, len);
}

int Telnet::read(char *buf, int len) {
  POFUNC;
  return ::read(sock, buf, len);
}

#endif /* USE_TELNET */
