/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * Xlib.cc - prototypes for X Window terminal driver.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef XLIB_H
#define XLIB_H

#ifdef USE_XLIB

#include <X11/Xlib.h>

#include "RawFont.h"
#include "CharMap.h"
#include "DisplayDriver.h"
#include "ConnectionDriver.h"

/// Generates a 16 bit colour X Window display.
class Xlib : public DisplayDriver {
  RawFont font;
  int depth; // Colour depth of display
  void *fbuf;
  int display_seq;
  void clear(void);
  void free(void);
  BOOL EnableFrameBuffer(BOOL enable);

  CharMap lastmap;

  /* X context data */
  Display *dpy;
  int scr;
  Colormap cmap;
  Window win;
  GC gc;
  int colourmap[16];
  XImage image;

  ///
  void HandleXEvents(ConnectionDriver &sd, TerminalDriver &);
  ///
  void OpenWindow(int width, int height, const String &title);
  ///
  void Draw(TerminalDriver &tty, BOOL refresh);

  ///
public:
  /// Default constructor.
  Xlib(void);
  /// Destructor.
  ~Xlib();

  /** Creates a new Xlib object with the appropriate attributes.
    @param arg height of display in pixels.
    @return NULL on failure, pointer to new TTYDisplay object otherwise.
    */
  friend Xlib *newXlib(const String &arg);

  /// (see DisplayDriver)
  void Main(ConnectionDriver &sd, TerminalDriver &terminal);
};

#endif /* USE_XLIB */

#endif /* XLIB_H */
